# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dgt_sdk/protobuf/client_status.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='dgt_sdk/protobuf/client_status.proto',
  package='',
  syntax='proto3',
  serialized_options=_b('\n\025sawtooth.sdk.protobufP\001Z\rclient_status'),
  serialized_pb=_b('\n$dgt_sdk/protobuf/client_status.proto\"\x18\n\x16\x43lientStatusGetRequest\"\xd3\x01\n\x17\x43lientStatusGetResponse\x12/\n\x06status\x18\x01 \x01(\x0e\x32\x1f.ClientStatusGetResponse.Status\x12,\n\x05peers\x18\x02 \x03(\x0b\x32\x1d.ClientStatusGetResponse.Peer\x12\x10\n\x08\x65ndpoint\x18\x03 \x01(\t\x1a\x18\n\x04Peer\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\t\"-\n\x06Status\x12\x10\n\x0cSTATUS_UNSET\x10\x00\x12\x06\n\x02OK\x10\x01\x12\t\n\x05\x45RROR\x10\x02\x42(\n\x15sawtooth.sdk.protobufP\x01Z\rclient_statusb\x06proto3')
)



_CLIENTSTATUSGETRESPONSE_STATUS = _descriptor.EnumDescriptor(
  name='Status',
  full_name='ClientStatusGetResponse.Status',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STATUS_UNSET', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OK', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=233,
  serialized_end=278,
)
_sym_db.RegisterEnumDescriptor(_CLIENTSTATUSGETRESPONSE_STATUS)


_CLIENTSTATUSGETREQUEST = _descriptor.Descriptor(
  name='ClientStatusGetRequest',
  full_name='ClientStatusGetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=40,
  serialized_end=64,
)


_CLIENTSTATUSGETRESPONSE_PEER = _descriptor.Descriptor(
  name='Peer',
  full_name='ClientStatusGetResponse.Peer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='ClientStatusGetResponse.Peer.endpoint', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=207,
  serialized_end=231,
)

_CLIENTSTATUSGETRESPONSE = _descriptor.Descriptor(
  name='ClientStatusGetResponse',
  full_name='ClientStatusGetResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='ClientStatusGetResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='peers', full_name='ClientStatusGetResponse.peers', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='ClientStatusGetResponse.endpoint', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CLIENTSTATUSGETRESPONSE_PEER, ],
  enum_types=[
    _CLIENTSTATUSGETRESPONSE_STATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=67,
  serialized_end=278,
)

_CLIENTSTATUSGETRESPONSE_PEER.containing_type = _CLIENTSTATUSGETRESPONSE
_CLIENTSTATUSGETRESPONSE.fields_by_name['status'].enum_type = _CLIENTSTATUSGETRESPONSE_STATUS
_CLIENTSTATUSGETRESPONSE.fields_by_name['peers'].message_type = _CLIENTSTATUSGETRESPONSE_PEER
_CLIENTSTATUSGETRESPONSE_STATUS.containing_type = _CLIENTSTATUSGETRESPONSE
DESCRIPTOR.message_types_by_name['ClientStatusGetRequest'] = _CLIENTSTATUSGETREQUEST
DESCRIPTOR.message_types_by_name['ClientStatusGetResponse'] = _CLIENTSTATUSGETRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ClientStatusGetRequest = _reflection.GeneratedProtocolMessageType('ClientStatusGetRequest', (_message.Message,), dict(
  DESCRIPTOR = _CLIENTSTATUSGETREQUEST,
  __module__ = 'dgt_sdk.protobuf.client_status_pb2'
  # @@protoc_insertion_point(class_scope:ClientStatusGetRequest)
  ))
_sym_db.RegisterMessage(ClientStatusGetRequest)

ClientStatusGetResponse = _reflection.GeneratedProtocolMessageType('ClientStatusGetResponse', (_message.Message,), dict(

  Peer = _reflection.GeneratedProtocolMessageType('Peer', (_message.Message,), dict(
    DESCRIPTOR = _CLIENTSTATUSGETRESPONSE_PEER,
    __module__ = 'dgt_sdk.protobuf.client_status_pb2'
    # @@protoc_insertion_point(class_scope:ClientStatusGetResponse.Peer)
    ))
  ,
  DESCRIPTOR = _CLIENTSTATUSGETRESPONSE,
  __module__ = 'dgt_sdk.protobuf.client_status_pb2'
  # @@protoc_insertion_point(class_scope:ClientStatusGetResponse)
  ))
_sym_db.RegisterMessage(ClientStatusGetResponse)
_sym_db.RegisterMessage(ClientStatusGetResponse.Peer)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dgt_sdk/protobuf/merkle.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='dgt_sdk/protobuf/merkle.proto',
  package='',
  syntax='proto3',
  serialized_options=_b('\n\025sawtooth.sdk.protobufP\001Z\nmerkle_pb2'),
  serialized_pb=_b('\n\x1d\x64gt_sdk/protobuf/merkle.proto\"\x95\x01\n\x0e\x43hangeLogEntry\x12\x0e\n\x06parent\x18\x01 \x01(\x0c\x12\x11\n\tadditions\x18\x02 \x03(\x0c\x12-\n\nsuccessors\x18\x03 \x03(\x0b\x32\x19.ChangeLogEntry.Successor\x1a\x31\n\tSuccessor\x12\x11\n\tsuccessor\x18\x01 \x01(\x0c\x12\x11\n\tdeletions\x18\x02 \x03(\x0c\x42%\n\x15sawtooth.sdk.protobufP\x01Z\nmerkle_pb2b\x06proto3')
)




_CHANGELOGENTRY_SUCCESSOR = _descriptor.Descriptor(
  name='Successor',
  full_name='ChangeLogEntry.Successor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='successor', full_name='ChangeLogEntry.Successor.successor', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deletions', full_name='ChangeLogEntry.Successor.deletions', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=134,
  serialized_end=183,
)

_CHANGELOGENTRY = _descriptor.Descriptor(
  name='ChangeLogEntry',
  full_name='ChangeLogEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='parent', full_name='ChangeLogEntry.parent', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='additions', full_name='ChangeLogEntry.additions', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='successors', full_name='ChangeLogEntry.successors', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CHANGELOGENTRY_SUCCESSOR, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=34,
  serialized_end=183,
)

_CHANGELOGENTRY_SUCCESSOR.containing_type = _CHANGELOGENTRY
_CHANGELOGENTRY.fields_by_name['successors'].message_type = _CHANGELOGENTRY_SUCCESSOR
DESCRIPTOR.message_types_by_name['ChangeLogEntry'] = _CHANGELOGENTRY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ChangeLogEntry = _reflection.GeneratedProtocolMessageType('ChangeLogEntry', (_message.Message,), dict(

  Successor = _reflection.GeneratedProtocolMessageType('Successor', (_message.Message,), dict(
    DESCRIPTOR = _CHANGELOGENTRY_SUCCESSOR,
    __module__ = 'dgt_sdk.protobuf.merkle_pb2'
    # @@protoc_insertion_point(class_scope:ChangeLogEntry.Successor)
    ))
  ,
  DESCRIPTOR = _CHANGELOGENTRY,
  __module__ = 'dgt_sdk.protobuf.merkle_pb2'
  # @@protoc_insertion_point(class_scope:ChangeLogEntry)
  ))
_sym_db.RegisterMessage(ChangeLogEntry)
_sym_db.RegisterMessage(ChangeLogEntry.Successor)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Copyright 2017 DGT NETWORK INC © Stanislav Parsov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------------------

import abc


class TransactionHandler(metaclass=abc.ABCMeta):
    """
    TransactionHandler is the Abstract Base Class that defines the business
    logic for a new transaction family.

    The family_name, family_versions, and namespaces properties are
    used by the processor to route processing requests to the handler.
    """

    @abc.abstractproperty
    def family_name(self):
        """
        family_name should return the name of the transaction family that this
        handler can process, e.g. "intkey"
        """
        pass

    @abc.abstractproperty
    def family_versions(self):
        """
        family_versions should return a list of versions this transaction
        family handler can process, e.g. ["1.0"]
        """
        pass

    @abc.abstractproperty
    def namespaces(self):
        """
        namespaces should return a list containing all the handler's
        namespaces, e.g. ["abcdef"]
        """
        pass

    @abc.abstractmethod
    def apply(self, transaction, context):
        """
        Apply is the single method where all the business logic for a
        transaction family is defined. The method will be called by the
        transaction processor upon receiving a TpProcessRequest that the
        handler understands and will pass in the TpProcessRequest and an
        initialized instance of the Context type.
        """
        pass



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dgt_sdk/protobuf/processor.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from dgt_sdk.protobuf import transaction_pb2 as dgt__sdk_dot_protobuf_dot_transaction__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='dgt_sdk/protobuf/processor.proto',
  package='',
  syntax='proto3',
  serialized_options=_b('\n\025sawtooth.sdk.protobufP\001Z\rprocessor_pb2'),
  serialized_pb=_b('\n dgt_sdk/protobuf/processor.proto\x1a\"dgt_sdk/protobuf/transaction.proto\"_\n\x11TpRegisterRequest\x12\x0e\n\x06\x66\x61mily\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x12\n\nnamespaces\x18\x04 \x03(\t\x12\x15\n\rmax_occupancy\x18\x05 \x01(\r\"o\n\x12TpRegisterResponse\x12*\n\x06status\x18\x01 \x01(\x0e\x32\x1a.TpRegisterResponse.Status\"-\n\x06Status\x12\x10\n\x0cSTATUS_UNSET\x10\x00\x12\x06\n\x02OK\x10\x01\x12\t\n\x05\x45RROR\x10\x02\"\x15\n\x13TpUnregisterRequest\"s\n\x14TpUnregisterResponse\x12,\n\x06status\x18\x01 \x01(\x0e\x32\x1c.TpUnregisterResponse.Status\"-\n\x06Status\x12\x10\n\x0cSTATUS_UNSET\x10\x00\x12\x06\n\x02OK\x10\x01\x12\t\n\x05\x45RROR\x10\x02\"n\n\x10TpProcessRequest\x12\"\n\x06header\x18\x01 \x01(\x0b\x32\x12.TransactionHeader\x12\x0f\n\x07payload\x18\x02 \x01(\x0c\x12\x11\n\tsignature\x18\x03 \x01(\t\x12\x12\n\ncontext_id\x18\x04 \x01(\t\"\xb7\x01\n\x11TpProcessResponse\x12)\n\x06status\x18\x01 \x01(\x0e\x32\x19.TpProcessResponse.Status\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x15\n\rextended_data\x18\x03 \x01(\x0c\"O\n\x06Status\x12\x10\n\x0cSTATUS_UNSET\x10\x00\x12\x06\n\x02OK\x10\x01\x12\x17\n\x13INVALID_TRANSACTION\x10\x02\x12\x12\n\x0eINTERNAL_ERROR\x10\x03\x42(\n\x15sawtooth.sdk.protobufP\x01Z\rprocessor_pb2b\x06proto3')
  ,
  dependencies=[dgt__sdk_dot_protobuf_dot_transaction__pb2.DESCRIPTOR,])



_TPREGISTERRESPONSE_STATUS = _descriptor.EnumDescriptor(
  name='Status',
  full_name='TpRegisterResponse.Status',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STATUS_UNSET', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OK', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=235,
  serialized_end=280,
)
_sym_db.RegisterEnumDescriptor(_TPREGISTERRESPONSE_STATUS)

_TPUNREGISTERRESPONSE_STATUS = _descriptor.EnumDescriptor(
  name='Status',
  full_name='TpUnregisterResponse.Status',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STATUS_UNSET', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OK', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=235,
  serialized_end=280,
)
_sym_db.RegisterEnumDescriptor(_TPUNREGISTERRESPONSE_STATUS)

_TPPROCESSRESPONSE_STATUS = _descriptor.EnumDescriptor(
  name='Status',
  full_name='TpProcessResponse.Status',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STATUS_UNSET', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OK', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INVALID_TRANSACTION', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INTERNAL_ERROR', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=639,
  serialized_end=718,
)
_sym_db.RegisterEnumDescriptor(_TPPROCESSRESPONSE_STATUS)


_TPREGISTERREQUEST = _descriptor.Descriptor(
  name='TpRegisterRequest',
  full_name='TpRegisterRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='family', full_name='TpRegisterRequest.family', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='TpRegisterRequest.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaces', full_name='TpRegisterRequest.namespaces', index=2,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_occupancy', full_name='TpRegisterRequest.max_occupancy', index=3,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=72,
  serialized_end=167,
)


_TPREGISTERRESPONSE = _descriptor.Descriptor(
  name='TpRegisterResponse',
  full_name='TpRegisterResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='TpRegisterResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TPREGISTERRESPONSE_STATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=169,
  serialized_end=280,
)


_TPUNREGISTERREQUEST = _descriptor.Descriptor(
  name='TpUnregisterRequest',
  full_name='TpUnregisterRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=282,
  serialized_end=303,
)


_TPUNREGISTERRESPONSE = _descriptor.Descriptor(
  name='TpUnregisterResponse',
  full_name='TpUnregisterResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='TpUnregisterResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TPUNREGISTERRESPONSE_STATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=305,
  serialized_end=420,
)


_TPPROCESSREQUEST = _descriptor.Descriptor(
  name='TpProcessRequest',
  full_name='TpProcessRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='TpProcessRequest.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload', full_name='TpProcessRequest.payload', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature', full_name='TpProcessRequest.signature', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='context_id', full_name='TpProcessRequest.context_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=422,
  serialized_end=532,
)


_TPPROCESSRESPONSE = _descriptor.Descriptor(
  name='TpProcessResponse',
  full_name='TpProcessResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='TpProcessResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='TpProcessResponse.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extended_data', full_name='TpProcessResponse.extended_data', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TPPROCESSRESPONSE_STATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=535,
  serialized_end=718,
)

_TPREGISTERRESPONSE.fields_by_name['status'].enum_type = _TPREGISTERRESPONSE_STATUS
_TPREGISTERRESPONSE_STATUS.containing_type = _TPREGISTERRESPONSE
_TPUNREGISTERRESPONSE.fields_by_name['status'].enum_type = _TPUNREGISTERRESPONSE_STATUS
_TPUNREGISTERRESPONSE_STATUS.containing_type = _TPUNREGISTERRESPONSE
_TPPROCESSREQUEST.fields_by_name['header'].message_type = dgt__sdk_dot_protobuf_dot_transaction__pb2._TRANSACTIONHEADER
_TPPROCESSRESPONSE.fields_by_name['status'].enum_type = _TPPROCESSRESPONSE_STATUS
_TPPROCESSRESPONSE_STATUS.containing_type = _TPPROCESSRESPONSE
DESCRIPTOR.message_types_by_name['TpRegisterRequest'] = _TPREGISTERREQUEST
DESCRIPTOR.message_types_by_name['TpRegisterResponse'] = _TPREGISTERRESPONSE
DESCRIPTOR.message_types_by_name['TpUnregisterRequest'] = _TPUNREGISTERREQUEST
DESCRIPTOR.message_types_by_name['TpUnregisterResponse'] = _TPUNREGISTERRESPONSE
DESCRIPTOR.message_types_by_name['TpProcessRequest'] = _TPPROCESSREQUEST
DESCRIPTOR.message_types_by_name['TpProcessResponse'] = _TPPROCESSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TpRegisterRequest = _reflection.GeneratedProtocolMessageType('TpRegisterRequest', (_message.Message,), dict(
  DESCRIPTOR = _TPREGISTERREQUEST,
  __module__ = 'dgt_sdk.protobuf.processor_pb2'
  # @@protoc_insertion_point(class_scope:TpRegisterRequest)
  ))
_sym_db.RegisterMessage(TpRegisterRequest)

TpRegisterResponse = _reflection.GeneratedProtocolMessageType('TpRegisterResponse', (_message.Message,), dict(
  DESCRIPTOR = _TPREGISTERRESPONSE,
  __module__ = 'dgt_sdk.protobuf.processor_pb2'
  # @@protoc_insertion_point(class_scope:TpRegisterResponse)
  ))
_sym_db.RegisterMessage(TpRegisterResponse)

TpUnregisterRequest = _reflection.GeneratedProtocolMessageType('TpUnregisterRequest', (_message.Message,), dict(
  DESCRIPTOR = _TPUNREGISTERREQUEST,
  __module__ = 'dgt_sdk.protobuf.processor_pb2'
  # @@protoc_insertion_point(class_scope:TpUnregisterRequest)
  ))
_sym_db.RegisterMessage(TpUnregisterRequest)

TpUnregisterResponse = _reflection.GeneratedProtocolMessageType('TpUnregisterResponse', (_message.Message,), dict(
  DESCRIPTOR = _TPUNREGISTERRESPONSE,
  __module__ = 'dgt_sdk.protobuf.processor_pb2'
  # @@protoc_insertion_point(class_scope:TpUnregisterResponse)
  ))
_sym_db.RegisterMessage(TpUnregisterResponse)

TpProcessRequest = _reflection.GeneratedProtocolMessageType('TpProcessRequest', (_message.Message,), dict(
  DESCRIPTOR = _TPPROCESSREQUEST,
  __module__ = 'dgt_sdk.protobuf.processor_pb2'
  # @@protoc_insertion_point(class_scope:TpProcessRequest)
  ))
_sym_db.RegisterMessage(TpProcessRequest)

TpProcessResponse = _reflection.GeneratedProtocolMessageType('TpProcessResponse', (_message.Message,), dict(
  DESCRIPTOR = _TPPROCESSRESPONSE,
  __module__ = 'dgt_sdk.protobuf.processor_pb2'
  # @@protoc_insertion_point(class_scope:TpProcessResponse)
  ))
_sym_db.RegisterMessage(TpProcessResponse)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Copyright 2016 DGT NETWORK INC © Stanislav Parsov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------------------

'''The processor module defines:

1. A TransactionHandler interface to be
used to create new transaction families.

2. A high-level, general purpose TransactionProcessor to which any
number of handlers can be added.

3. A Context class used to abstract getting and setting addresses in
global validator state.
'''

__all__ = [
    'core',
    'context',
    'exceptions'
]



# Copyright 2016 DGT NETWORK INC © Stanislav Parsov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------------------

import asyncio
import uuid
import logging
from queue import Queue
from threading import Event
from threading import Thread
from threading import Condition

import zmq
import zmq.asyncio

from dgt_sdk.protobuf import validator_pb2

from dgt_sdk.messaging.exceptions import ValidatorConnectionError
from dgt_sdk.messaging.future import Future
from dgt_sdk.messaging.future import FutureCollection
from dgt_sdk.messaging.future import FutureCollectionKeyError
from dgt_sdk.messaging.future import FutureResult
from dgt_sdk.messaging.future import FutureError

LOGGER = logging.getLogger(__file__)

# Used to send a message to core.TransactionProcessor to reregister with
# the validator and wait for TP_PROCESS_REQUEST. All other items on the
# queue will be validator_pb2.Message objects, so -1 will be an exceptional
# event.
RECONNECT_EVENT = -1
_NO_ERROR = -1


def _generate_id():
    return uuid.uuid4().hex.encode()


class _SendReceiveThread(Thread):
    """
    Internal thread to Stream class that runs the asyncio event loop.
    """

    def __init__(self, url, futures, ready_event, error_queue):
        """constructor for background thread

        :param url (str): the address to connect to the validator on
        :param futures (FutureCollection): The Futures associated with
                messages sent through Stream.send
        :param ready_event (threading.Event): used to notify waiting/asking
               classes that the background thread of Stream is ready after
               a disconnect event.
        """
        super(_SendReceiveThread, self).__init__()
        self._futures = futures
        self._url = url
        self._shutdown = False
        self._event_loop = None
        self._sock = None
        self._monitor_sock = None
        self._monitor_fd = None
        self._recv_queue = None
        self._send_queue = None
        self._context = None
        self._ready_event = ready_event
        self._error_queue = error_queue
        self._condition = Condition()
        self.identity = _generate_id()[0:16]

    @asyncio.coroutine
    def _receive_message(self):
        """
        internal coroutine that receives messages and puts
        them on the recv_queue
        """
        while True:
            if not self._ready_event.is_set():
                break
            msg_bytes = yield from self._sock.recv()
            message = validator_pb2.Message()
            message.ParseFromString(msg_bytes)
            try:
                self._futures.set_result(
                    message.correlation_id,
                    FutureResult(message_type=message.message_type,
                                 content=message.content))
                self._futures.remove(message.correlation_id)
            except FutureCollectionKeyError:
                # if we are getting an initial message, not a response
                if not self._ready_event.is_set():
                    break
                self._recv_queue.put_nowait(message)

    @asyncio.coroutine
    def _send_message(self):
        """
        internal coroutine that sends messages from the send_queue
        """
        while True:
            if not self._ready_event.is_set():
                break
            msg = yield from self._send_queue.get()
            yield from self._sock.send_multipart([msg.SerializeToString()])

    @asyncio.coroutine
    def _put_message(self, message):
        """
        Puts a message on the send_queue. Not to be accessed directly.
        :param message: protobuf generated validator_pb2.Message
        """
        self._send_queue.put_nowait(message)

    @asyncio.coroutine
    def _get_message(self):
        """
        Gets a message from the recv_queue. Not to be accessed directly.
        """
        with self._condition:
            self._condition.wait_for(lambda: self._recv_queue is not None)
        msg = yield from self._recv_queue.get()

        return msg

    @asyncio.coroutine
    def _monitor_disconnects(self):
        """Monitors the client socket for disconnects
        """
        yield from self._monitor_sock.recv_multipart()
        self._sock.disable_monitor()
        self._monitor_sock.disconnect(self._monitor_fd)
        self._monitor_sock.close(linger=0)
        self._monitor_sock = None
        self._sock.disconnect(self._url)
        self._ready_event.clear()
        LOGGER.debug("monitor socket received disconnect event")
        for future in self._futures.future_values():
            future.set_result(FutureError())
        tasks = list(asyncio.Task.all_tasks(self._event_loop))
        for task in tasks:
            task.cancel()
        self._event_loop.stop()
        self._send_queue = None
        self._recv_queue = None

    def put_message(self, message):
        """
        :param message: protobuf generated validator_pb2.Message
        """
        if not self._ready_event.is_set():
            return

        with self._condition:
            self._condition.wait_for(
                lambda: self._event_loop is not None
                and self._send_queue is not None
            )

        asyncio.run_coroutine_threadsafe(
            self._put_message(message),
            self._event_loop)

    def get_message(self):
        """
        :return message: concurrent.futures.Future
        """
        with self._condition:
            self._condition.wait_for(lambda: self._event_loop is not None)
        return asyncio.run_coroutine_threadsafe(self._get_message(),
                                                self._event_loop)

    def _cancel_tasks_yet_to_be_done(self):
        """Cancels all the tasks (pending coroutines and futures)
        """
        tasks = list(asyncio.Task.all_tasks(self._event_loop))
        for task in tasks:
            self._event_loop.call_soon_threadsafe(task.cancel)
        self._event_loop.call_soon_threadsafe(self._done_callback)

    def shutdown(self):
        """Shutdown the _SendReceiveThread. Is an irreversible operation.
        """

        self._shutdown = True
        self._cancel_tasks_yet_to_be_done()

    def _done_callback(self):
        """Stops the event loop, closes the socket, and destroys the context

        :param future: concurrent.futures.Future not used
        """
        self._event_loop.call_soon_threadsafe(self._event_loop.stop)
        self._sock.close(linger=0)
        self._monitor_sock.close(linger=0)
        self._context.destroy(linger=0)

    def run(self):
        first_time = True
        while True:
            try:
                if self._event_loop is None:
                    self._event_loop = zmq.asyncio.ZMQEventLoop()
                    asyncio.set_event_loop(self._event_loop)
                if self._context is None:
                    self._context = zmq.asyncio.Context()
                if self._sock is None:
                    self._sock = self._context.socket(zmq.DEALER)
                self._sock.identity = self.identity

                self._sock.connect(self._url)

                self._monitor_fd = "inproc://monitor.s-{}".format(
                    _generate_id()[0:5])
                self._monitor_sock = self._sock.get_monitor_socket(
                    zmq.EVENT_DISCONNECTED,
                    addr=self._monitor_fd)
                self._send_queue = asyncio.Queue(loop=self._event_loop)
                self._recv_queue = asyncio.Queue(loop=self._event_loop)
                if first_time is False:
                    self._recv_queue.put_nowait(RECONNECT_EVENT)
                with self._condition:
                    self._condition.notify_all()
                asyncio.ensure_future(self._send_message(),
                                      loop=self._event_loop)
                asyncio.ensure_future(self._receive_message(),
                                      loop=self._event_loop)
                asyncio.ensure_future(self._monitor_disconnects(),
                                      loop=self._event_loop)
                # pylint: disable=broad-except
            except Exception as e:
                LOGGER.error("Exception connecting to validator "
                             "address %s, so shutting down", self._url)
                self._error_queue.put_nowait(e)
                break

            self._error_queue.put_nowait(_NO_ERROR)
            self._ready_event.set()
            self._event_loop.run_forever()
            if self._shutdown:
                self._sock.close(linger=0)
                self._monitor_sock.close(linger=0)
                self._context.destroy(linger=0)
                break
            if first_time is True:
                first_time = False


class Stream:
    def __init__(self, url):
        self._url = url
        self._futures = FutureCollection()
        self._event = Event()
        self._event.set()
        error_queue = Queue()
        self._send_recieve_thread = _SendReceiveThread(
            url,
            futures=self._futures,
            ready_event=self._event,
            error_queue=error_queue)
        self._send_recieve_thread.start()
        err = error_queue.get()
        if err is not _NO_ERROR:
            raise err

    @property
    def url(self):
        """ Get the url of the Stream object.
        """
        return self._url

    @property
    def zmq_id(self):
        return self._send_recieve_thread.identity

    def send(self, message_type, content):
        """Send a message to the validator

        :param: message_type(validator_pb2.Message.MessageType)
        :param: content(bytes)
        :return: (future.Future)
        :raises: (ValidatorConnectionError)
        """

        if not self._event.is_set():
            raise ValidatorConnectionError()
        message = validator_pb2.Message(
            message_type=message_type,
            correlation_id=_generate_id(),
            content=content)
        future = Future(message.correlation_id, request_type=message_type)
        self._futures.put(future)

        self._send_recieve_thread.put_message(message)
        return future

    def send_back(self, message_type, correlation_id, content):
        """
        Return a response to a message.
        :param message_type: validator_pb2.Message.MessageType enum value
        :param correlation_id: a random str internal to the validator
        :param content: protobuf bytes
        :raises (ValidatorConnectionError):
        """
        if not self._event.is_set():
            raise ValidatorConnectionError()
        message = validator_pb2.Message(
            message_type=message_type,
            correlation_id=correlation_id,
            content=content)
        self._send_recieve_thread.put_message(message)

    def receive(self):
        """
        Receive messages that are not responses
        :return: concurrent.futures.Future
        """
        return self._send_recieve_thread.get_message()

    def wait_for_ready(self):
        """Blocks until the background thread has recovered
        from a disconnect with the validator.
        """
        self._event.wait()

    def is_ready(self):
        """Whether the background thread has recovered from
        a disconnect with the validator

        :return: (bool) whether the background thread is ready
        """
        return self._event.is_set()

    def close(self):
        self._send_recieve_thread.shutdown()



# Copyright 2016, 2017 DGT NETWORK INC © Stanislav Parsov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------------------
import os
from dgt_signing.core import NoSuchAlgorithmError
from dgt_signing.core import ParseError
from dgt_signing.core import SigningError

from dgt_signing.secp256k1 import Secp256k1Context
from dgt_signing.open_crypto import OpenCryptoContext

DGT_CRYPTO_NM = 'dgt.crypto'
DGT_CRYPTO_ALG_NM = 'dgt.crypto.alg'









CRYPTO_BACK = os.environ.get('CRYPTO_BACK')
class Signer:
    """A convenient wrapper of Context and PrivateKey
    """

    def __init__(self, context, private_key):
        """
        """
        self._context = context
        self._private_key = private_key
        self._public_key = None

    def sign(self, message):
        """Signs the given message

        Args:
            message (bytes): the message bytes

        Returns:
            The signature in a hex-encoded string

        Raises:
            SigningError: if any error occurs during the signing process
        """
        return self._context.sign(message, self._private_key)

    @property
    def private_key(self):
        return self._private_key

    @property
    def context(self):
        return self._context

    def verify(self, signature, message, public):
        return self._context.verify(signature,message,public)

    def get_public_key(self):
        """Return the public key for this Signer instance.
        """
        # Lazy-eval the public key
        if self._public_key is None:
            self._public_key = self._context.get_public_key(self._private_key)
        return self._public_key


class CryptoFactory:
    """Factory for generating Signers.
    """

    def __init__(self, context):
        self._context = context

    @property
    def context(self):
        """Return the context that backs this factory instance
        """
        return self._context

    def new_signer(self, private_key):
        """Create a new signer for the given private key.

        Args:
            private_key (:obj:`PrivateKey`): a private key

        Returns:
            (:obj:`Signer`): a signer instance
        """
        return Signer(self._context, private_key)


def create_context(algorithm_name,backend='bitcoin'):
    """Returns an algorithm instance by name.

    Args:
        algorithm_name (str): the algorithm name

    Returns:
        (:obj:`Context`): a context instance for the given algorithm

    Raises:
        NoSuchAlgorithmError if the algorithm is unknown
    """
    if CRYPTO_BACK != '' and CRYPTO_BACK is not None:
        backend = CRYPTO_BACK
    if backend == 'bitcoin':
        # old version
        if algorithm_name == 'secp256k1':
            return Secp256k1Context()
    elif backend == 'openssl':
        # openssl version
        return OpenCryptoContext(algorithm=algorithm_name)

    raise NoSuchAlgorithmError(f"no such algorithm: {algorithm_name} for backend={backend}")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dgt_sdk/protobuf/transaction.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='dgt_sdk/protobuf/transaction.proto',
  package='',
  syntax='proto3',
  serialized_options=_b('\n\025sawtooth.sdk.protobufP\001Z\017transaction_pb2'),
  serialized_pb=_b('\n\"dgt_sdk/protobuf/transaction.proto\"\xd5\x01\n\x11TransactionHeader\x12\x1a\n\x12\x62\x61tcher_public_key\x18\x01 \x01(\t\x12\x14\n\x0c\x64\x65pendencies\x18\x02 \x03(\t\x12\x13\n\x0b\x66\x61mily_name\x18\x03 \x01(\t\x12\x16\n\x0e\x66\x61mily_version\x18\x04 \x01(\t\x12\x0e\n\x06inputs\x18\x05 \x03(\t\x12\r\n\x05nonce\x18\x06 \x01(\t\x12\x0f\n\x07outputs\x18\x07 \x03(\t\x12\x16\n\x0epayload_sha512\x18\t \x01(\t\x12\x19\n\x11signer_public_key\x18\n \x01(\t\"H\n\x0bTransaction\x12\x0e\n\x06header\x18\x01 \x01(\x0c\x12\x18\n\x10header_signature\x18\x02 \x01(\t\x12\x0f\n\x07payload\x18\x03 \x01(\x0c\"5\n\x0fTransactionList\x12\"\n\x0ctransactions\x18\x01 \x03(\x0b\x32\x0c.TransactionB*\n\x15sawtooth.sdk.protobufP\x01Z\x0ftransaction_pb2b\x06proto3')
)




_TRANSACTIONHEADER = _descriptor.Descriptor(
  name='TransactionHeader',
  full_name='TransactionHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='batcher_public_key', full_name='TransactionHeader.batcher_public_key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dependencies', full_name='TransactionHeader.dependencies', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='family_name', full_name='TransactionHeader.family_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='family_version', full_name='TransactionHeader.family_version', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='inputs', full_name='TransactionHeader.inputs', index=4,
      number=5, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nonce', full_name='TransactionHeader.nonce', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='outputs', full_name='TransactionHeader.outputs', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload_sha512', full_name='TransactionHeader.payload_sha512', index=7,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signer_public_key', full_name='TransactionHeader.signer_public_key', index=8,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=39,
  serialized_end=252,
)


_TRANSACTION = _descriptor.Descriptor(
  name='Transaction',
  full_name='Transaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='Transaction.header', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='header_signature', full_name='Transaction.header_signature', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload', full_name='Transaction.payload', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=254,
  serialized_end=326,
)


_TRANSACTIONLIST = _descriptor.Descriptor(
  name='TransactionList',
  full_name='TransactionList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='transactions', full_name='TransactionList.transactions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=328,
  serialized_end=381,
)

_TRANSACTIONLIST.fields_by_name['transactions'].message_type = _TRANSACTION
DESCRIPTOR.message_types_by_name['TransactionHeader'] = _TRANSACTIONHEADER
DESCRIPTOR.message_types_by_name['Transaction'] = _TRANSACTION
DESCRIPTOR.message_types_by_name['TransactionList'] = _TRANSACTIONLIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TransactionHeader = _reflection.GeneratedProtocolMessageType('TransactionHeader', (_message.Message,), dict(
  DESCRIPTOR = _TRANSACTIONHEADER,
  __module__ = 'dgt_sdk.protobuf.transaction_pb2'
  # @@protoc_insertion_point(class_scope:TransactionHeader)
  ))
_sym_db.RegisterMessage(TransactionHeader)

Transaction = _reflection.GeneratedProtocolMessageType('Transaction', (_message.Message,), dict(
  DESCRIPTOR = _TRANSACTION,
  __module__ = 'dgt_sdk.protobuf.transaction_pb2'
  # @@protoc_insertion_point(class_scope:Transaction)
  ))
_sym_db.RegisterMessage(Transaction)

TransactionList = _reflection.GeneratedProtocolMessageType('TransactionList', (_message.Message,), dict(
  DESCRIPTOR = _TRANSACTIONLIST,
  __module__ = 'dgt_sdk.protobuf.transaction_pb2'
  # @@protoc_insertion_point(class_scope:TransactionList)
  ))
_sym_db.RegisterMessage(TransactionList)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Copyright 2018 DGT NETWORK INC © Stanislav Parsov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -----------------------------------------------------------------------------

import concurrent
import logging
from queue import Queue
from threading import Thread

from dgt_sdk.consensus.driver import Driver
from dgt_sdk.consensus.engine import StartupState
from dgt_sdk.consensus.zmq_service import ZmqService
from dgt_sdk.consensus import exceptions
from dgt_sdk.messaging.stream import Stream
from dgt_sdk.protobuf import consensus_pb2
from dgt_sdk.protobuf.validator_pb2 import Message


LOGGER = logging.getLogger(__name__)
REGISTER_TIMEOUT = 800
SERVICE_TIMEOUT = 800


class ZmqDriver(Driver):
    def __init__(self, engine):
        super().__init__(engine)
        self._engine = engine
        self._stream = None
        self._exit = False
        self._updates = None
        self._signed_consensus = self._engine.signed_consensus

    def start(self, endpoint):
        LOGGER.debug('ZmqDriver: start endpoint=%s',endpoint)
        self._stream = Stream(endpoint)

        startup_state = self._register()

        self._updates = Queue()

        driver_thread = Thread(
            target=self._driver_loop)
        driver_thread.start()

        try:
            self._engine.start(
                self._updates,
                ZmqService(
                    stream=self._stream,
                    timeout=SERVICE_TIMEOUT,
                    name=self._engine.name(),
                    version=self._engine.version()),
                startup_state)
        except Exception as ex:  # pylint: disable=broad-except
            LOGGER.exception("Uncaught engine exception(%s)",ex)

        self.stop()
        driver_thread.join()

    def _driver_loop(self):
        try:
            future = self._stream.receive()
            LOGGER.debug('ZmqDriver: _driver_loop future=%s',future)
            while True:
                if self._exit:
                    self._engine.stop()
                    break

                try:
                    message = future.result(1)
                    future = self._stream.receive()
                except concurrent.futures.TimeoutError:
                    continue
                #LOGGER.debug('ZmqDriver: _driver_loop _process')
                result = self._process(message)

                self._updates.put(result)
        except Exception:  # pylint: disable=broad-except
            LOGGER.exception("Uncaught driver exception")

    def stop(self):
        self._exit = True
        self._engine.stop()
        self._stream.close()

    def _register(self):
        self._stream.wait_for_ready()

        request = consensus_pb2.ConsensusRegisterRequest(
            name=self._engine.name(),
            version=self._engine.version(),
        ).SerializeToString()

        while True:
            future = self._stream.send(
                message_type=Message.CONSENSUS_REGISTER_REQUEST,
                content=request)
            response = consensus_pb2.ConsensusRegisterResponse()
            response.ParseFromString(future.result(REGISTER_TIMEOUT).content)

            if (
                response.status
                == consensus_pb2.ConsensusRegisterResponse.NOT_READY
            ):
                #LOGGER.debug('ZmqDriver:register NOT_READY: url=%s',self._stream._url)
                continue

            if response.status == consensus_pb2.ConsensusRegisterResponse.OK :
                LOGGER.debug('ZmqDriver:register DONE: url=%s',self._stream._url)
                return StartupState(
                    response.chain_head,
                    response.peers,
                    response.local_peer_info,
                    response.peering_mode)

            raise exceptions.ReceiveError(
                'Registration failed with status {}'.format(response.status))

    def _process(self, message):
        type_tag = message.message_type

        if type_tag == Message.CONSENSUS_NOTIFY_PEER_CONNECTED:
            notification = consensus_pb2.ConsensusNotifyPeerConnected()
            notification.ParseFromString(message.content)

            data = notification.peer_info, notification.status, notification.mode , notification.info, notification.data

        elif type_tag == Message.CONSENSUS_NOTIFY_PEER_DISCONNECTED:
            notification = consensus_pb2.ConsensusNotifyPeerDisconnected()
            notification.ParseFromString(message.content)

            data = notification.peer_id

        elif type_tag == Message.CONSENSUS_NOTIFY_PEER_MESSAGE:
            notification = consensus_pb2.ConsensusNotifyPeerMessageNew() if self._signed_consensus else consensus_pb2.ConsensusNotifyPeerMessage()
            notification.ParseFromString(message.content)

            data = notification.message, notification.sender_id

        elif type_tag == Message.CONSENSUS_NOTIFY_BLOCK_NEW:
            notification = consensus_pb2.ConsensusNotifyBlockNew()
            notification.ParseFromString(message.content)

            data = notification.block

        elif type_tag == Message.CONSENSUS_NOTIFY_BLOCK_VALID:
            notification = consensus_pb2.ConsensusNotifyBlockValid()
            notification.ParseFromString(message.content)

            data = notification.block_id

        elif type_tag == Message.CONSENSUS_NOTIFY_BLOCK_INVALID:
            notification = consensus_pb2.ConsensusNotifyBlockInvalid()
            notification.ParseFromString(message.content)

            data = notification.block_id

        elif type_tag == Message.CONSENSUS_NOTIFY_BLOCK_COMMIT:
            notification = consensus_pb2.ConsensusNotifyBlockCommit()
            notification.ParseFromString(message.content)

            data = notification.block_id

        else:
            raise exceptions.ReceiveError(
                'Received unexpected message type: {}'.format(type_tag))

        self._stream.send_back(
            message_type=Message.CONSENSUS_NOTIFY_ACK,
            correlation_id=message.correlation_id,
            content=consensus_pb2.ConsensusNotifyAck().SerializeToString())

        return type_tag, data



# Copyright 2017 DGT NETWORK INC © Stanislav Parsov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------------------
import os
import sys
import toml
import yaml


def get_config_dir():
    """Returns the sawtooth configuration directory based on the
    SAWTOOTH_HOME environment variable (if set) or OS defaults.
    """
    if 'PEER_HOME' in os.environ:
        return os.path.join(os.environ['PEER_HOME'], 'etc')
    elif 'SAWTOOTH_HOME' in os.environ:
        return os.path.join(os.environ['SAWTOOTH_HOME'], 'etc')

    if os.name == 'nt':
        base_dir = \
            os.path.dirname(os.path.dirname(os.path.abspath(sys.argv[0])))
        return os.path.join(base_dir, 'conf')

    return '/etc/sawtooth'


def _get_dir(toml_config_setting,
             sawtooth_home_dir,
             windows_dir,
             default_dir):
    """Determines the directory path based on configuration.

    Arguments:
        toml_config_setting (str): The name of the config setting related
            to the directory which will appear in path.toml.
        sawtooth_home_dir (str): The directory under the SAWTOOTH_HOME
            environment variable.  For example, for 'data' if the data
            directory is $SAWTOOTH_HOME/data.
        windows_dir (str): The windows path relative to the computed base
            directory.
        default_dir (str): The default path on Linux.

    Returns:
        directory (str): The path.
    """
    conf_file = os.path.join(get_config_dir(), 'path.toml')
    if os.path.exists(conf_file):
        with open(conf_file) as fd:
            raw_config = fd.read()
        toml_config = toml.loads(raw_config)
        if toml_config_setting in toml_config:
            return toml_config[toml_config_setting]

    if 'PEER_HOME' in os.environ:
        return os.path.join(os.environ['PEER_HOME'], sawtooth_home_dir)
    elif 'SAWTOOTH_HOME' in os.environ:
        return os.path.join(os.environ['SAWTOOTH_HOME'], sawtooth_home_dir)

    if os.name == 'nt':
        base_dir = \
            os.path.dirname(os.path.dirname(os.path.abspath(sys.argv[0])))
        return os.path.join(base_dir, windows_dir)

    return default_dir


def _get_log_config(filename=None):
    """Determines if there is a log config in the config directory
       and returns it. If it does not exist, return None.

    Arguments:
        filename (str): The name of the logging config specific to the
            transaction processor that is being started.

    Returns:
        log_config (dict): The dictionary to pass to logging.config.dictConfig
    """
    if filename is not None:

        conf_file = os.path.join(get_config_dir(), filename)
        if os.path.exists(conf_file):
            with open(conf_file) as fd:
                raw_config = fd.read()
            if filename.endswith(".yaml"):
                log_config = yaml.safe_load(raw_config)
            else:
                log_config = toml.loads(raw_config)
            return log_config
    return None


def _get_processor_config(filename=None):
    """Determines if there is a proccesor config in the config directory
       and returns it. If it does not exist, return None.

    Arguments:
        filename (str): The name of the processor config specific to the
            transaction processor that is being started.

    Returns:
        processor_config (dict): The dictionary to set transaction processor
    """

    if filename is not None:

        conf_file = os.path.join(get_config_dir(), filename)
        if os.path.exists(conf_file):
            with open(conf_file) as fd:
                raw_config = fd.read()
                log_config = toml.loads(raw_config)
                return log_config
    return None


def get_log_dir():
    """Returns the configured data directory."""
    return _get_dir(
        toml_config_setting='log_dir',
        sawtooth_home_dir='logs',
        windows_dir='logs',
        default_dir='/var/log/sawtooth')


def get_log_config(filename=None):
    """Returns the log config dictinary if it exists."""
    return _get_log_config(filename)


def get_processor_config(filename=None):
    """Returns the log config dictinary if it exists."""
    return _get_processor_config(filename)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dgt_sdk/protobuf/batch.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from dgt_sdk.protobuf import transaction_pb2 as dgt__sdk_dot_protobuf_dot_transaction__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='dgt_sdk/protobuf/batch.proto',
  package='',
  syntax='proto3',
  serialized_options=_b('\n\025sawtooth.sdk.protobufP\001Z\tbatch_pb2'),
  serialized_pb=_b('\n\x1c\x64gt_sdk/protobuf/batch.proto\x1a\"dgt_sdk/protobuf/transaction.proto\"A\n\x0b\x42\x61tchHeader\x12\x19\n\x11signer_public_key\x18\x01 \x01(\t\x12\x17\n\x0ftransaction_ids\x18\x02 \x03(\t\"w\n\x05\x42\x61tch\x12\x0e\n\x06header\x18\x01 \x01(\x0c\x12\x18\n\x10header_signature\x18\x02 \x01(\t\x12\"\n\x0ctransactions\x18\x03 \x03(\x0b\x32\x0c.Transaction\x12\r\n\x05trace\x18\x04 \x01(\x08\x12\x11\n\ttimestamp\x18\x05 \x01(\r\"M\n\tBatchList\x12\x17\n\x07\x62\x61tches\x18\x01 \x03(\x0b\x32\x06.Batch\x12\x14\n\x0c\x63\x61ndidate_id\x18\x02 \x01(\x0c\x12\x11\n\tblock_num\x18\x03 \x01(\rB$\n\x15sawtooth.sdk.protobufP\x01Z\tbatch_pb2b\x06proto3')
  ,
  dependencies=[dgt__sdk_dot_protobuf_dot_transaction__pb2.DESCRIPTOR,])




_BATCHHEADER = _descriptor.Descriptor(
  name='BatchHeader',
  full_name='BatchHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='signer_public_key', full_name='BatchHeader.signer_public_key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='transaction_ids', full_name='BatchHeader.transaction_ids', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=68,
  serialized_end=133,
)


_BATCH = _descriptor.Descriptor(
  name='Batch',
  full_name='Batch',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='Batch.header', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='header_signature', full_name='Batch.header_signature', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='transactions', full_name='Batch.transactions', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='trace', full_name='Batch.trace', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='Batch.timestamp', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=135,
  serialized_end=254,
)


_BATCHLIST = _descriptor.Descriptor(
  name='BatchList',
  full_name='BatchList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='batches', full_name='BatchList.batches', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='candidate_id', full_name='BatchList.candidate_id', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='block_num', full_name='BatchList.block_num', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=256,
  serialized_end=333,
)

_BATCH.fields_by_name['transactions'].message_type = dgt__sdk_dot_protobuf_dot_transaction__pb2._TRANSACTION
_BATCHLIST.fields_by_name['batches'].message_type = _BATCH
DESCRIPTOR.message_types_by_name['BatchHeader'] = _BATCHHEADER
DESCRIPTOR.message_types_by_name['Batch'] = _BATCH
DESCRIPTOR.message_types_by_name['BatchList'] = _BATCHLIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BatchHeader = _reflection.GeneratedProtocolMessageType('BatchHeader', (_message.Message,), dict(
  DESCRIPTOR = _BATCHHEADER,
  __module__ = 'dgt_sdk.protobuf.batch_pb2'
  # @@protoc_insertion_point(class_scope:BatchHeader)
  ))
_sym_db.RegisterMessage(BatchHeader)

Batch = _reflection.GeneratedProtocolMessageType('Batch', (_message.Message,), dict(
  DESCRIPTOR = _BATCH,
  __module__ = 'dgt_sdk.protobuf.batch_pb2'
  # @@protoc_insertion_point(class_scope:Batch)
  ))
_sym_db.RegisterMessage(Batch)

BatchList = _reflection.GeneratedProtocolMessageType('BatchList', (_message.Message,), dict(
  DESCRIPTOR = _BATCHLIST,
  __module__ = 'dgt_sdk.protobuf.batch_pb2'
  # @@protoc_insertion_point(class_scope:BatchList)
  ))
_sym_db.RegisterMessage(BatchList)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dgt_sdk/protobuf/events.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='dgt_sdk/protobuf/events.proto',
  package='',
  syntax='proto3',
  serialized_options=_b('\n\025sawtooth.sdk.protobufP\001Z\nevents_pb2'),
  serialized_pb=_b('\n\x1d\x64gt_sdk/protobuf/events.proto\"x\n\x05\x45vent\x12\x12\n\nevent_type\x18\x01 \x01(\t\x12$\n\nattributes\x18\x02 \x03(\x0b\x32\x10.Event.Attribute\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x1a\'\n\tAttribute\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"#\n\tEventList\x12\x16\n\x06\x65vents\x18\x01 \x03(\x0b\x32\x06.Event\"\xc1\x01\n\x0b\x45ventFilter\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x14\n\x0cmatch_string\x18\x02 \x01(\t\x12,\n\x0b\x66ilter_type\x18\x03 \x01(\x0e\x32\x17.EventFilter.FilterType\"a\n\nFilterType\x12\x15\n\x11\x46ILTER_TYPE_UNSET\x10\x00\x12\x0e\n\nSIMPLE_ANY\x10\x01\x12\x0e\n\nSIMPLE_ALL\x10\x02\x12\r\n\tREGEX_ANY\x10\x03\x12\r\n\tREGEX_ALL\x10\x04\"F\n\x11\x45ventSubscription\x12\x12\n\nevent_type\x18\x01 \x01(\t\x12\x1d\n\x07\x66ilters\x18\x02 \x03(\x0b\x32\x0c.EventFilterB%\n\x15sawtooth.sdk.protobufP\x01Z\nevents_pb2b\x06proto3')
)



_EVENTFILTER_FILTERTYPE = _descriptor.EnumDescriptor(
  name='FilterType',
  full_name='EventFilter.FilterType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FILTER_TYPE_UNSET', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SIMPLE_ANY', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SIMPLE_ALL', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REGEX_ANY', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REGEX_ALL', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=289,
  serialized_end=386,
)
_sym_db.RegisterEnumDescriptor(_EVENTFILTER_FILTERTYPE)


_EVENT_ATTRIBUTE = _descriptor.Descriptor(
  name='Attribute',
  full_name='Event.Attribute',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='Event.Attribute.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='Event.Attribute.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=114,
  serialized_end=153,
)

_EVENT = _descriptor.Descriptor(
  name='Event',
  full_name='Event',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='event_type', full_name='Event.event_type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributes', full_name='Event.attributes', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='Event.data', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_EVENT_ATTRIBUTE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=33,
  serialized_end=153,
)


_EVENTLIST = _descriptor.Descriptor(
  name='EventList',
  full_name='EventList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='events', full_name='EventList.events', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=155,
  serialized_end=190,
)


_EVENTFILTER = _descriptor.Descriptor(
  name='EventFilter',
  full_name='EventFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='EventFilter.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='match_string', full_name='EventFilter.match_string', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filter_type', full_name='EventFilter.filter_type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _EVENTFILTER_FILTERTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=193,
  serialized_end=386,
)


_EVENTSUBSCRIPTION = _descriptor.Descriptor(
  name='EventSubscription',
  full_name='EventSubscription',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='event_type', full_name='EventSubscription.event_type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filters', full_name='EventSubscription.filters', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=388,
  serialized_end=458,
)

_EVENT_ATTRIBUTE.containing_type = _EVENT
_EVENT.fields_by_name['attributes'].message_type = _EVENT_ATTRIBUTE
_EVENTLIST.fields_by_name['events'].message_type = _EVENT
_EVENTFILTER.fields_by_name['filter_type'].enum_type = _EVENTFILTER_FILTERTYPE
_EVENTFILTER_FILTERTYPE.containing_type = _EVENTFILTER
_EVENTSUBSCRIPTION.fields_by_name['filters'].message_type = _EVENTFILTER
DESCRIPTOR.message_types_by_name['Event'] = _EVENT
DESCRIPTOR.message_types_by_name['EventList'] = _EVENTLIST
DESCRIPTOR.message_types_by_name['EventFilter'] = _EVENTFILTER
DESCRIPTOR.message_types_by_name['EventSubscription'] = _EVENTSUBSCRIPTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Event = _reflection.GeneratedProtocolMessageType('Event', (_message.Message,), dict(

  Attribute = _reflection.GeneratedProtocolMessageType('Attribute', (_message.Message,), dict(
    DESCRIPTOR = _EVENT_ATTRIBUTE,
    __module__ = 'dgt_sdk.protobuf.events_pb2'
    # @@protoc_insertion_point(class_scope:Event.Attribute)
    ))
  ,
  DESCRIPTOR = _EVENT,
  __module__ = 'dgt_sdk.protobuf.events_pb2'
  # @@protoc_insertion_point(class_scope:Event)
  ))
_sym_db.RegisterMessage(Event)
_sym_db.RegisterMessage(Event.Attribute)

EventList = _reflection.GeneratedProtocolMessageType('EventList', (_message.Message,), dict(
  DESCRIPTOR = _EVENTLIST,
  __module__ = 'dgt_sdk.protobuf.events_pb2'
  # @@protoc_insertion_point(class_scope:EventList)
  ))
_sym_db.RegisterMessage(EventList)

EventFilter = _reflection.GeneratedProtocolMessageType('EventFilter', (_message.Message,), dict(
  DESCRIPTOR = _EVENTFILTER,
  __module__ = 'dgt_sdk.protobuf.events_pb2'
  # @@protoc_insertion_point(class_scope:EventFilter)
  ))
_sym_db.RegisterMessage(EventFilter)

EventSubscription = _reflection.GeneratedProtocolMessageType('EventSubscription', (_message.Message,), dict(
  DESCRIPTOR = _EVENTSUBSCRIPTION,
  __module__ = 'dgt_sdk.protobuf.events_pb2'
  # @@protoc_insertion_point(class_scope:EventSubscription)
  ))
_sym_db.RegisterMessage(EventSubscription)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Copyright 2016, 2017 DGT NETWORK INC © Stanislav Parsov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------------------

from abc import ABCMeta
from abc import abstractmethod

X509_COUNTRY_NAME           = "COUNTRY_NAME"   
X509_STATE_OR_PROVINCE_NAME = "STATE_OR_PROVINCE_NAME"   
X509_LOCALITY_NAME          = "LOCALITY_NAME"   
X509_ORGANIZATION_NAME      = "ORGANIZATION_NAME"   
X509_COMMON_NAME            = "COMMON_NAME"   
X509_DNS_NAME               = "DNS_NAME"
X509_EMAIL_ADDRESS          = "EMAIL_ADDRESS" 
X509_PSEUDONYM              = "PSEUDONYM"     


class NoSuchAlgorithmError(Exception):
    """Thrown when trying to create an algorithm which does not exist.
    """
    pass


class SigningError(Exception):
    """Thrown when an error occurs during the signing process.
    """
    pass


class ParseError(Exception):
    """Thrown when an error occurs during deserialization of a Private or
    Public key from various formats.
    """
    pass


class PrivateKey(metaclass=ABCMeta):
    """A private key instance.

    The underlying content is dependent on implementation.
    """

    @abstractmethod
    def get_algorithm_name(self):
        """Returns the algorithm name used for this private key.
        """
        pass

    @abstractmethod
    def as_hex(self):
        """Return the private key encoded as a hex string.
        """
        pass

    @abstractmethod
    def as_bytes(self):
        """Return the private key bytes.
        """
        pass


class PublicKey(metaclass=ABCMeta):
    """A public key instance.

    The underlying content is dependent on implementation.
    """

    @abstractmethod
    def get_algorithm_name(self):
        """Returns the algorithm name used for this public key.
        """
        pass

    @abstractmethod
    def as_hex(self):
        """Return the public key encoded as a hex string.
        """
        pass

    @abstractmethod
    def as_bytes(self):
        """Return the public key bytes.
        """
        pass


class Context(metaclass=ABCMeta):
    """A context for a cryptographic signing algorithm.
    """

    @abstractmethod
    def get_algorithm_name(self):
        """Returns the algorithm name.
        """
        pass

    @abstractmethod
    def sign(self, message, private_key):
        """Sign a message

        Given a private key for this algorithm, sign the given message bytes
        and return a hex-encoded string of the resulting signature.

        Args:
            message (bytes): the message bytes
            private_key (:obj:`PrivateKey`): the private key

        Returns:
            The signature in a hex-encoded string

        Raises:
            SigningError: if any error occurs during the signing process
        """
        pass

    @abstractmethod
    def verify(self, signature, message, public_key):
        """Verifies that a signature of a message was produced with the
        associated public key.

        Args:
            signature (str): the hex-encoded signature
            message (bytes): the message bytes
            public_key (:obj:`PublicKey`): the public key to use for
                verification

        Returns:
            boolean: True if the public key is associated with the signature
            for that method, False otherwise
        """

    @abstractmethod
    def new_random_private_key(self):
        """Generates a new random PrivateKey using this context.

        Returns:
            (:obj:`PrivateKey`): a random private key
        """

    @abstractmethod
    def get_public_key(self, private_key):
        """Produce a public key for the given private key.

        Args:
            private_key (:obj:`PrivateKey`): a private key

        Returns:
            (:obj:`PublicKey`) the public key for the given private key
        """
        pass



# Copyright 2016, 2017 DGT NETWORK INC © Stanislav Parsov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------------------

import binascii
import warnings

import secp256k1

from dgt_signing.core import SigningError
from dgt_signing.core import ParseError

from dgt_signing.core import PrivateKey
from dgt_signing.core import PublicKey
from dgt_signing.core import Context
import logging


class Secp256k1PrivateKey(PrivateKey):
    def __init__(self, secp256k1_private_key):
        self._private_key = secp256k1_private_key

    def get_algorithm_name(self):
        return "secp256k1"

    def as_hex(self):
        return binascii.hexlify(self.as_bytes()).decode()

    def as_bytes(self):
        return bytes(self._private_key.private_key)

    @property
    def secp256k1_private_key(self):
        return self._private_key

    @staticmethod
    def from_wif(hex_str):
        raise ParseError(f'Unable to parse hex private key: {hex_str}')

    @staticmethod
    def from_hex(hex_str):
        try:
            priv = binascii.unhexlify(hex_str)
            logging.info(f"Secp256k1PrivateKey.from_hex: hex={hex_str} serialized={priv}")
            return Secp256k1PrivateKey(secp256k1.PrivateKey(priv, None))
        except Exception as e:
            raise ParseError(f'Unable to parse hex private key={hex_str}: {e}')

    @staticmethod
    def new_random():
        return Secp256k1PrivateKey(secp256k1.PrivateKey(None))


class Secp256k1PublicKey(PublicKey):
    def __init__(self, secp256k1_public_key):
        self._public_key = secp256k1_public_key

    @property
    def secp256k1_public_key(self):
        return self._public_key

    def get_algorithm_name(self):
        return "secp256k1"

    def as_hex(self):
        return binascii.hexlify(self.as_bytes()).decode()

    def as_bytes(self):
        with warnings.catch_warnings():  # squelch secp256k1 warning
            warnings.simplefilter('ignore')
            return self._public_key.serialize()

    @staticmethod
    def from_hex(hex_str):
        try:
            public_key = __PK__.deserialize(binascii.unhexlify(hex_str))

            return Secp256k1PublicKey(
                secp256k1.PublicKey(public_key, None ))
        except Exception as e:
            raise ParseError('Unable to parse public key: {}'.format(e))


class Secp256k1Context(Context):
    def __init__(self):
        self._ctx = None

    def get_algorithm_name(self):
        return "secp256k1"

    def sign(self, message, private_key):
        try:
            signature = private_key.secp256k1_private_key.ecdsa_sign(message)
            signature = private_key.secp256k1_private_key.ecdsa_serialize_compact(signature)

            return signature.hex()
        except Exception as e:
            raise SigningError('Unable to sign message: {}'.format(str(e)))

    def verify(self, signature, message, public_key):
        try:
            sig_bytes = bytes.fromhex(signature)

            sig = public_key.secp256k1_public_key.ecdsa_deserialize_compact(sig_bytes)
            return public_key.secp256k1_public_key.ecdsa_verify(message, sig)
        # pylint: disable=broad-except
        except Exception:
            return False

    def new_random_private_key(self):
        return Secp256k1PrivateKey.new_random()

    def new_random(self):           
        return Secp256k1PrivateKey.new_random() 

    def get_public_key(self, private_key):
        return Secp256k1PublicKey(private_key.secp256k1_private_key.pubkey)

    def from_hex(self,hex_str):
        return Secp256k1PrivateKey.from_hex(hex_str)

    def from_wif(self,hex_str):
        return Secp256k1PrivateKey.from_wif(hex_str)

    def pub_from_hex(self,hex_str):
        return Secp256k1PublicKey.from_hex(hex_str)

    def create_x509_certificate(self,subject_info,priv,after=None,before=None):
        return self.sign(b'xxxxxx',priv).encode('utf-8')

    def load_x509_certificate(self,cert_pem):                                            
        xcert = {}
        
        return xcert                                                                     



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dgt_sdk/protobuf/identity.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='dgt_sdk/protobuf/identity.proto',
  package='',
  syntax='proto3',
  serialized_options=_b('\n\032sawtooth.identity.protobufP\001'),
  serialized_pb=_b('\n\x1f\x64gt_sdk/protobuf/identity.proto\"\xae\x01\n\x06Policy\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1e\n\x07\x65ntries\x18\x02 \x03(\x0b\x32\r.Policy.Entry\x1a\x35\n\x05\x45ntry\x12\x1f\n\x04type\x18\x01 \x01(\x0e\x32\x11.Policy.EntryType\x12\x0b\n\x03key\x18\x02 \x01(\t\"?\n\tEntryType\x12\x14\n\x10\x45NTRY_TYPE_UNSET\x10\x00\x12\x0e\n\nPERMIT_KEY\x10\x01\x12\x0c\n\x08\x44\x45NY_KEY\x10\x02\"\'\n\nPolicyList\x12\x19\n\x08policies\x18\x01 \x03(\x0b\x32\x07.Policy\")\n\x04Role\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0bpolicy_name\x18\x02 \x01(\t\" \n\x08RoleList\x12\x14\n\x05roles\x18\x01 \x03(\x0b\x32\x05.RoleB\x1e\n\x1asawtooth.identity.protobufP\x01\x62\x06proto3')
)



_POLICY_ENTRYTYPE = _descriptor.EnumDescriptor(
  name='EntryType',
  full_name='Policy.EntryType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ENTRY_TYPE_UNSET', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PERMIT_KEY', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DENY_KEY', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=147,
  serialized_end=210,
)
_sym_db.RegisterEnumDescriptor(_POLICY_ENTRYTYPE)


_POLICY_ENTRY = _descriptor.Descriptor(
  name='Entry',
  full_name='Policy.Entry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='Policy.Entry.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='key', full_name='Policy.Entry.key', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=92,
  serialized_end=145,
)

_POLICY = _descriptor.Descriptor(
  name='Policy',
  full_name='Policy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Policy.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entries', full_name='Policy.entries', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_POLICY_ENTRY, ],
  enum_types=[
    _POLICY_ENTRYTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=36,
  serialized_end=210,
)


_POLICYLIST = _descriptor.Descriptor(
  name='PolicyList',
  full_name='PolicyList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='policies', full_name='PolicyList.policies', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=212,
  serialized_end=251,
)


_ROLE = _descriptor.Descriptor(
  name='Role',
  full_name='Role',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Role.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='policy_name', full_name='Role.policy_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=253,
  serialized_end=294,
)


_ROLELIST = _descriptor.Descriptor(
  name='RoleList',
  full_name='RoleList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='roles', full_name='RoleList.roles', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=296,
  serialized_end=328,
)

_POLICY_ENTRY.fields_by_name['type'].enum_type = _POLICY_ENTRYTYPE
_POLICY_ENTRY.containing_type = _POLICY
_POLICY.fields_by_name['entries'].message_type = _POLICY_ENTRY
_POLICY_ENTRYTYPE.containing_type = _POLICY
_POLICYLIST.fields_by_name['policies'].message_type = _POLICY
_ROLELIST.fields_by_name['roles'].message_type = _ROLE
DESCRIPTOR.message_types_by_name['Policy'] = _POLICY
DESCRIPTOR.message_types_by_name['PolicyList'] = _POLICYLIST
DESCRIPTOR.message_types_by_name['Role'] = _ROLE
DESCRIPTOR.message_types_by_name['RoleList'] = _ROLELIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Policy = _reflection.GeneratedProtocolMessageType('Policy', (_message.Message,), dict(

  Entry = _reflection.GeneratedProtocolMessageType('Entry', (_message.Message,), dict(
    DESCRIPTOR = _POLICY_ENTRY,
    __module__ = 'dgt_sdk.protobuf.identity_pb2'
    # @@protoc_insertion_point(class_scope:Policy.Entry)
    ))
  ,
  DESCRIPTOR = _POLICY,
  __module__ = 'dgt_sdk.protobuf.identity_pb2'
  # @@protoc_insertion_point(class_scope:Policy)
  ))
_sym_db.RegisterMessage(Policy)
_sym_db.RegisterMessage(Policy.Entry)

PolicyList = _reflection.GeneratedProtocolMessageType('PolicyList', (_message.Message,), dict(
  DESCRIPTOR = _POLICYLIST,
  __module__ = 'dgt_sdk.protobuf.identity_pb2'
  # @@protoc_insertion_point(class_scope:PolicyList)
  ))
_sym_db.RegisterMessage(PolicyList)

Role = _reflection.GeneratedProtocolMessageType('Role', (_message.Message,), dict(
  DESCRIPTOR = _ROLE,
  __module__ = 'dgt_sdk.protobuf.identity_pb2'
  # @@protoc_insertion_point(class_scope:Role)
  ))
_sym_db.RegisterMessage(Role)

RoleList = _reflection.GeneratedProtocolMessageType('RoleList', (_message.Message,), dict(
  DESCRIPTOR = _ROLELIST,
  __module__ = 'dgt_sdk.protobuf.identity_pb2'
  # @@protoc_insertion_point(class_scope:RoleList)
  ))
_sym_db.RegisterMessage(RoleList)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Copyright 2016 DGT NETWORK INC © Stanislav Parsov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------------------

from concurrent.futures import CancelledError
import concurrent.futures
import itertools
import logging


from dgt_sdk.messaging.exceptions import ValidatorConnectionError
from dgt_sdk.messaging.future import FutureTimeoutError
from dgt_sdk.messaging.stream import RECONNECT_EVENT
from dgt_sdk.messaging.stream import Stream

from dgt_sdk.processor.context import Context
from dgt_sdk.processor.exceptions import InvalidTransaction
from dgt_sdk.processor.exceptions import InternalError
from dgt_sdk.processor.exceptions import AuthorizationException

from dgt_sdk.protobuf.processor_pb2 import TpRegisterRequest
from dgt_sdk.protobuf.processor_pb2 import TpRegisterResponse
from dgt_sdk.protobuf.processor_pb2 import TpUnregisterRequest
from dgt_sdk.protobuf.processor_pb2 import TpUnregisterResponse
from dgt_sdk.protobuf.processor_pb2 import TpProcessRequest
from dgt_sdk.protobuf.processor_pb2 import TpProcessResponse
from dgt_sdk.protobuf.network_pb2 import PingResponse
from dgt_sdk.protobuf.validator_pb2 import Message


LOGGER = logging.getLogger(__name__)


class TransactionProcessor:
    """TransactionProcessor is a generic class for communicating with a
    validator and routing transaction processing requests to a registered
    handler. It uses ZMQ and channels to handle requests concurrently.
    """

    def __init__(self, url):
        """
        Args:
            url (string): The URL of the validator
        """
        self._stream = Stream(url)
        self._url = url
        self._handlers = []

    @property
    def zmq_id(self):
        return self._stream.zmq_id

    def add_handler(self, handler):
        """Adds a transaction family handler
        Args:
            handler (TransactionHandler): the handler to be added
        """
        self._handlers.append(handler)

    def _matches(self, handler, header):
        LOGGER.debug("Matches handler: %s~%s", header.family_name,handler.family_name)
        return header.family_name == handler.family_name \
            and header.family_version in handler.family_versions

    def _find_handler(self, header):
        """Find a handler for a particular (family_name, family_versions)
        :param header transaction_pb2.TransactionHeader:
        :return: handler
        """
        LOGGER.debug("find_handler...")
        try:
            return next(
                handler for handler in self._handlers
                if self._matches(handler, header))
        except StopIteration:
            LOGGER.debug("Missing handler for header: %s", header)
            return None

    def _register_requests(self):
        """Returns all of the TpRegisterRequests for handlers

        :return (list): list of TpRegisterRequests
        """
        return itertools.chain.from_iterable(  # flattens the nested list
            [
                [TpRegisterRequest(
                    family=n,
                    version=v,
                    namespaces=h.namespaces)
                 for n, v in itertools.product(
                    [h.family_name],
                     h.family_versions,)] for h in self._handlers])

    def _unregister_request(self):
        """Returns a single TP_UnregisterRequest that requests
        that the validator stop sending transactions for previously
        registered handlers.

        :return (processor_pb2.TpUnregisterRequest):
        """
        return TpUnregisterRequest()

    def _process(self, msg):
        if msg.message_type != Message.TP_PROCESS_REQUEST:
            LOGGER.debug(
                "Transaction Processor recieved invalid message type. "
                "Message type should be TP_PROCESS_REQUEST,"
                " but is %s", Message.MessageType.Name(msg.message_type))
            return
        LOGGER.debug("Transaction Processor: process %s.",Message.MessageType.Name(msg.message_type))
        request = TpProcessRequest()
        request.ParseFromString(msg.content)
        state = Context(self._stream, request.context_id)
        header = request.header
        try:
            if not self._stream.is_ready():
                raise ValidatorConnectionError()
            handler = self._find_handler(header)
            if handler is None:
                return
            LOGGER.debug('TransactionProcessor: _process.apply context_id=%s',request.context_id)
            handler.apply(request, state)
            self._stream.send_back(
                message_type=Message.TP_PROCESS_RESPONSE,
                correlation_id=msg.correlation_id,
                content=TpProcessResponse(
                    status=TpProcessResponse.OK
                ).SerializeToString())
        except InvalidTransaction as it:
            LOGGER.warning("Invalid Transaction %s", it)
            try:
                self._stream.send_back(
                    message_type=Message.TP_PROCESS_RESPONSE,
                    correlation_id=msg.correlation_id,
                    content=TpProcessResponse(
                        status=TpProcessResponse.INVALID_TRANSACTION,
                        message=str(it),
                        extended_data=it.extended_data
                    ).SerializeToString())
            except ValidatorConnectionError as vce:
                # TP_PROCESS_REQUEST has made it through the
                # handler.apply and an INVALID_TRANSACTION would have been
                # sent back but the validator has disconnected and so it
                # doesn't care about the response.
                LOGGER.warning("during invalid transaction response: %s", vce)
        except InternalError as ie:
            LOGGER.warning("internal error: %s", ie)
            try:
                self._stream.send_back(
                    message_type=Message.TP_PROCESS_RESPONSE,
                    correlation_id=msg.correlation_id,
                    content=TpProcessResponse(
                        status=TpProcessResponse.INTERNAL_ERROR,
                        message=str(ie),
                        extended_data=ie.extended_data
                    ).SerializeToString())
            except ValidatorConnectionError as vce:
                # Same as the prior except block, but an internal error has
                # happened, but because of the disconnect the validator
                # probably doesn't care about the response.
                LOGGER.warning("during internal error response: %s", vce)
        except ValidatorConnectionError as vce:
            # Somewhere within handler.apply a future resolved with an
            # error status that the validator has disconnected. There is
            # nothing left to do but reconnect.
            LOGGER.warning("during handler.apply a future was resolved "
                           "with error status: %s", vce)
        except AuthorizationException as ae:
            LOGGER.warning("AuthorizationException: %s", ae)
            try:
                self._stream.send_back(
                    message_type=Message.TP_PROCESS_RESPONSE,
                    correlation_id=msg.correlation_id,
                    content=TpProcessResponse(
                        status=TpProcessResponse.INVALID_TRANSACTION,
                        message=str(ae),
                    ).SerializeToString())
            except ValidatorConnectionError as vce:
                # TP_PROCESS_REQUEST has made it through the
                # handler.apply and an INVALID_TRANSACTION would have been
                # sent back but the validator has disconnected and so it
                # doesn't care about the response.
                LOGGER.warning("during invalid transaction response: %s", vce)

    def _process_future(self, future, timeout=None, sigint=False):
        try:
            LOGGER.debug('TransactionProcessor: future.result ...',)
            msg = future.result(timeout)
        except CancelledError:
            # This error is raised when Task.cancel is called on
            # disconnect from the validator in stream.py, for
            # this future.
            LOGGER.debug('TransactionProcessor: CancelledError')
            return
        LOGGER.debug('TransactionProcessor: _process_future msg=%s',str(msg))
        if msg is RECONNECT_EVENT:
            if sigint is False:
                LOGGER.info("reregistering with validator")
                self._stream.wait_for_ready()
                self._register()
        else:
            LOGGER.debug(
                'received message of type: %s',
                Message.MessageType.Name(msg.message_type))
            if msg.message_type == Message.PING_REQUEST:
                self._stream.send_back(
                    message_type=Message.PING_RESPONSE,
                    correlation_id=msg.correlation_id,
                    content=PingResponse().SerializeToString())
                return
            self._process(msg)

    def _register(self):
        futures = []
        for message in self._register_requests():
            self._stream.wait_for_ready()
            future = self._stream.send(
                message_type=Message.TP_REGISTER_REQUEST,
                content=message.SerializeToString())
            futures.append(future)

        for future in futures:
            resp = TpRegisterResponse()
            try:
                resp.ParseFromString(future.result().content)
                LOGGER.info("Register attempt: %s",
                            TpRegisterResponse.Status.Name(resp.status))
            except ValidatorConnectionError as vce:
                LOGGER.info("during waiting for response on registration: %s",
                            vce)

    def _unregister(self):
        message = self._unregister_request()
        self._stream.wait_for_ready()
        future = self._stream.send(
            message_type=Message.TP_UNREGISTER_REQUEST,
            content=message.SerializeToString())
        response = TpUnregisterResponse()
        try:
            response.ParseFromString(future.result(1).content)
            LOGGER.info("unregister attempt: %s",
                        TpUnregisterResponse.Status.Name(response.status))
        except ValidatorConnectionError as vce:
            LOGGER.info("during waiting for response on unregistration: %s",
                        vce)

    def start(self):
        """Connects the transaction processor to a validator and starts
        listening for requests and routing them to an appropriate
        transaction handler.
        """
        fut = None
        try:
            self._register()
            while True:
                # During long running processing this
                # is where the transaction processor will
                # spend most of its time
                fut = self._stream.receive()
                LOGGER.debug("TransactionProcessor:receive: fut=%s", str(fut))
                self._process_future(fut)
        except KeyboardInterrupt:
            try:
                # tell the validator to not send any more messages
                self._unregister()
                while True:
                    if fut is not None:
                        # process futures as long as the tp has them,
                        # if the TP_PROCESS_REQUEST doesn't come from
                        # zeromq->asyncio in 1 second raise a
                        # concurrent.futures.TimeOutError and be done.
                        self._process_future(fut, 1, sigint=True)
                        fut = self._stream.receive()
            except concurrent.futures.TimeoutError:
                # Where the tp will usually exit after
                # a KeyboardInterrupt. Caused by the 1 second
                # timeout in _process_future.
                pass
            except FutureTimeoutError:
                # If the validator is not able to respond to the
                # unregister request, exit.
                pass

    def stop(self):
        """Closes the connection between the TransactionProcessor and the
        validator.
        """
        self._stream.close()



# Copyright 2018 DGT NETWORK INC © Stanislav Parsov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -----------------------------------------------------------------------------

from dgt_sdk.consensus.service import Service
from dgt_sdk.consensus.service import Block
from dgt_sdk.consensus import exceptions
from dgt_sdk.protobuf import consensus_pb2
from dgt_sdk.protobuf.validator_pb2 import Message
from dgt_sdk.protobuf.consensus_pb2 import ConsensusPeerMessageNew,ConsensusPeerMessageHeader
import logging
import hashlib
LOGGER = logging.getLogger(__name__)

class ZmqService(Service):
    def __init__(self, stream, timeout, name, version):
        self._stream = stream
        self._timeout = timeout
        self._name = name
        self._version = version
        self._signer = None 
    
    def set_signer(self,signer):
        # for signed consensus message
        self._signer = signer
        self._signer_public_key=self._signer.get_public_key().as_hex()
        LOGGER.debug(f"ZmqService:set_signer {self._signer_public_key}")

    def _send(self, request, message_type, response_type):
        response_bytes = self._stream.send(
            message_type=message_type,
            content=request.SerializeToString(),
        ).result(self._timeout).content

        response = response_type()
        response.ParseFromString(response_bytes)

        return response

    # -- P2P --
    def make_consensus_peer_message(self,message_type, payload):
        if self._signer is not None:
            # SIGNED MESSAGE MODE
            header = ConsensusPeerMessageHeader(                                          
                         signer_id = self._signer_public_key,                             
                         content_sha512  = hashlib.sha512(payload).hexdigest(),           
                         message_type = message_type,                                     
                         name = self._name,                                               
                         version = self._version                                          
                )                                                                         
            ser_header = header.SerializeToString()                                       
            signed_msg = ConsensusPeerMessageNew(                                               
                         header = ser_header,                                             
                         header_signature = self._signer.sign(ser_header),                
                         content = payload                                                
                )                                                                         
            #LOGGER.debug(f"ZmqService:consensus_peer_message header={header} msg={signed_msg}") 
            return signed_msg

        message = consensus_pb2.ConsensusPeerMessage(         
            message_type=message_type,                        
            content=payload,                                  
            name=self._name,                                  
            version=self._version)                            
        
        
         
        return message

    def send_to(self, peer_id, message_type, payload):
        message = self.make_consensus_peer_message(message_type, payload) 

        request = consensus_pb2.ConsensusSendToRequest(
            message=message.SerializeToString(),
            peer_id=peer_id)

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_SEND_TO_REQUEST,
            response_type=consensus_pb2.ConsensusSendToResponse)

        if response.status != consensus_pb2.ConsensusSendToResponse.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(response.status))

    def broadcast_to_cluster(self, message_type, payload):
        """
        FIXME - better use special message like CONSENSUS_BROADCAST_REQUEST and send only one message and validator take cluster's peer from topology
        """
        
        message = self.make_consensus_peer_message(message_type, payload)

        request = consensus_pb2.ConsensusBroadcastClusterRequest(message=message.SerializeToString())
        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_BROADCAST_CLUSTER_REQUEST,
            response_type=consensus_pb2.ConsensusBroadcastClusterResponse)

        if response.status != consensus_pb2.ConsensusBroadcastClusterResponse.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(response.status))
        

    def broadcast_to_arbiter(self, message_type, payload):
        """
        FIXME - better use special message like CONSENSUS_BROADCAST_REQUEST and send only one message and validator take cluster's peer from topology
        """
        message = self.make_consensus_peer_message(message_type, payload)

        request = consensus_pb2.ConsensusBroadcastArbiterRequest(message=message.SerializeToString())
        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_BROADCAST_ARBITER_REQUEST,
            response_type=consensus_pb2.ConsensusBroadcastArbiterResponse)

        if response.status != consensus_pb2.ConsensusBroadcastArbiterResponse.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(response.status))
        

    def broadcast(self, message_type, payload):
        message = self.make_consensus_peer_message(message_type, payload)

        request = consensus_pb2.ConsensusBroadcastRequest(message=message.SerializeToString())

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_BROADCAST_REQUEST,
            response_type=consensus_pb2.ConsensusBroadcastResponse)

        if response.status != consensus_pb2.ConsensusBroadcastResponse.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(response.status))

    # -- Block Creation --

    def initialize_block(self, previous_id=None,nest_colour=None):
        request = (
            consensus_pb2.ConsensusInitializeBlockRequest(
                previous_id=previous_id,nest_colour=nest_colour)
            if previous_id
            else consensus_pb2.ConsensusInitializeBlockRequest()
        )

        response_type = consensus_pb2.ConsensusInitializeBlockResponse

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_INITIALIZE_BLOCK_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.INVALID_STATE:
            raise exceptions.InvalidState('Cannot initialize block in current state')

        if status == response_type.UNKNOWN_BLOCK:
            raise exceptions.UnknownBlock()

        if status != response_type.OK:
            raise exceptions.ReceiveError(
                'Failed with status {}'.format(status))

    def summarize_block(self):
        request = consensus_pb2.ConsensusSummarizeBlockRequest()

        response_type = consensus_pb2.ConsensusSummarizeBlockResponse

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_SUMMARIZE_BLOCK_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.INVALID_STATE:
            raise exceptions.InvalidState('Cannot summarize block in current state')

        if status == response_type.BLOCK_NOT_READY:
            raise exceptions.BlockNotReady('Block not ready to be summarize')

        if status != response_type.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(status))

        return (response.summary,response.block_id)

    def finalize_block(self,block_id, data):
        request = consensus_pb2.ConsensusFinalizeBlockRequest(data=data,block_id=block_id)

        response_type = consensus_pb2.ConsensusFinalizeBlockResponse

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_FINALIZE_BLOCK_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.INVALID_STATE:
            raise exceptions.InvalidState('Cannot finalize block in current state')

        if status == response_type.BLOCK_NOT_READY:
            raise exceptions.BlockNotReady('Block not ready to be finalized')

        if status != response_type.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(status))

        return response.block_id

    def cancel_block(self,branch_id=None):
        request = consensus_pb2.ConsensusCancelBlockRequest(branch_id=branch_id)

        response_type = consensus_pb2.ConsensusCancelBlockResponse

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_CANCEL_BLOCK_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.INVALID_STATE:
            raise exceptions.InvalidState('Cannot cancel block in current state')

        if status != response_type.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(status))

    # -- Block Directives --

    def check_blocks(self, priority):
        request = consensus_pb2.ConsensusCheckBlocksRequest(block_ids=priority)

        response_type = consensus_pb2.ConsensusCheckBlocksResponse

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_CHECK_BLOCKS_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.UNKNOWN_BLOCK:
            raise exceptions.UnknownBlock()

        if status != response_type.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(status))

    def commit_block(self, block_id,seal=None):
        # place for SEAL
        request = consensus_pb2.ConsensusCommitBlockRequest(block_id=block_id,seal=seal.SerializeToString() if seal is not None else seal)

        response_type = consensus_pb2.ConsensusCommitBlockResponse

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_COMMIT_BLOCK_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.UNKNOWN_BLOCK:
            raise exceptions.UnknownBlock()

        if status != response_type.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(status))

    def ignore_block(self, block_id):
        request = consensus_pb2.ConsensusIgnoreBlockRequest(block_id=block_id)

        response_type = consensus_pb2.ConsensusIgnoreBlockResponse

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_IGNORE_BLOCK_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.UNKNOWN_BLOCK:
            raise exceptions.UnknownBlock()

        if status != response_type.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(status))

    def fail_block(self, block_id):
        request = consensus_pb2.ConsensusFailBlockRequest(block_id=block_id)

        response_type = consensus_pb2.ConsensusFailBlockResponse

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_FAIL_BLOCK_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.UNKNOWN_BLOCK:
            raise exceptions.UnknownBlock()

        if status != response_type.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(status))

    # -- Queries --

    def get_blocks(self, block_ids):
        request = consensus_pb2.ConsensusBlocksGetRequest(block_ids=block_ids)

        response_type = consensus_pb2.ConsensusBlocksGetResponse

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_BLOCKS_GET_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.UNKNOWN_BLOCK:
            raise exceptions.UnknownBlock()

        if status != response_type.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(status))

        return {
            block.block_id: Block(block)
            for block in response.blocks
        }

    def get_chain_head(self,parent_id = None,new_parent_id=None,is_new=False):
        request = consensus_pb2.ConsensusChainHeadGetRequest(parent_id=parent_id,new_parent_id=new_parent_id,is_new=is_new)

        response_type = consensus_pb2.ConsensusChainHeadGetResponse

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_CHAIN_HEAD_GET_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.NO_CHAIN_HEAD:
            raise exceptions.NoChainHead()
        if status == response_type.TOO_MANY_BRANCH:
            raise exceptions.TooManyBranch()
        if status == response_type.BLOCK_IS_PROCESSED_NOW:
            raise exceptions.BlockIsProcessedNow()
        if status != response_type.OK:
            raise exceptions.ReceiveError('Failed with status {}'.format(status))

        #LOGGER.debug('get_chain_head: block=%s',response.block)
        return Block(response.block)

    def get_settings(self, block_id, settings):
        request = consensus_pb2.ConsensusSettingsGetRequest(
            block_id=block_id,
            keys=settings)

        response_type = consensus_pb2.ConsensusSettingsGetResponse

        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_SETTINGS_GET_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.UNKNOWN_BLOCK:
            raise exceptions.UnknownBlock()

        if status != response_type.OK:
            raise exceptions.ReceiveError(f'Failed with status {status}')

        return {
            entry.key: entry.value
            for entry in response.entries
        }

    def get_state(self, block_id, addresses):
        request = consensus_pb2.ConsensusStateGetRequest(
            block_id=block_id,
            addresses=addresses)

        response_type = consensus_pb2.ConsensusStateGetResponse
        #LOGGER.debug(f"ZmqService:get_state {block_id}")
        response = self._send(
            request=request,
            message_type=Message.CONSENSUS_STATE_GET_REQUEST,
            response_type=response_type)

        status = response.status

        if status == response_type.UNKNOWN_BLOCK:
            raise exceptions.UnknownBlock()
        if status == response_type.BLOCK_IS_PROCESSED_NOW:   
            raise exceptions.BlockIsProcessedNow()           
        if status != response_type.OK:
            raise exceptions.ReceiveError(f'Failed with status {status}')

        return {
            entry.address: entry.data
            for entry in response.entries
        }



import socket
import sys
import os
import os.path

from dgt_signing import create_context
from dgt_signing import CryptoFactory
from dgt_sdk.protobuf.transaction_pb2 import TransactionHeader
from dgt_sdk.protobuf.transaction_pb2 import Transaction
from dgt_sdk.protobuf.transaction_pb2 import TransactionList
from dgt_sdk.protobuf.batch_pb2 import BatchList
from dgt_sdk.protobuf.batch_pb2 import BatchHeader
from dgt_sdk.protobuf.batch_pb2 import Batch

import json
import cbor
from hashlib import sha512
import random
import time
import requests



def send_request(url, suffix, data = None , content_type = None, name = None):
    headers={}
    try:
        headers['Content-Type'] = content_type
        result = requests.post(url, headers= headers , data=data)
    except Exception as e:
        print("Something went wrong: "+ str(e)) 
    return result.text

def _sha512(data):
    return sha512(data).hexdigest()

def get_prefix():
    return _sha512('bgt'.encode('utf-8'))[0:6]

def get_address(name):
    prefix = get_prefix()
    game_address=_sha512(name.encode('utf-8'))[64:]
    return prefix + game_address


def connect(host:str,port:int) -> bool:
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        res = s.connect_ex((host,port))
        return res

def send(IP,verb,wal,to,value,PK) -> str:
    value=int(value)
    URL = f"http://{IP}/batches"

    context=create_context('secp256k1',backend='openssl')
    PK = context.from_hex(PK)
    signer = CryptoFactory(context).new_signer(PK)


    payload = {'Name' : wal , 'Value' : value , 'Verb' : verb }

    address= get_address(wal)
    inputs = [address]
    outputs = [address]

    if to is not None:      
        payload["To"] = to
        inputs.append(get_address(to))
        outputs.append(get_address(to))

    payload_bytes = cbor.dumps(payload)
    transaction_header_bytes=TransactionHeader(
            batcher_public_key = signer.get_public_key().as_hex(),
            dependencies=[],
            family_name = "bgt" ,
            family_version = "1.0" ,
            inputs = inputs,
            outputs = outputs,
            nonce = hex(random.randint(0,2**64)),
            payload_sha512=_sha512(payload_bytes),
            signer_public_key=signer.get_public_key().as_hex()
            ).SerializeToString()

    signature = signer.sign(transaction_header_bytes)
    txn = Transaction(
            header = transaction_header_bytes,
            payload=payload_bytes,
            header_signature = signature
            )
    txns = [txn]
    batch_header_bytes = BatchHeader(
            signer_public_key=signer.get_public_key().as_hex(),
            transaction_ids = [txn.header_signature for txn in txns]
            ).SerializeToString()

    batch_signature = signer.sign(batch_header_bytes)

    batch = Batch(
            header = batch_header_bytes,
            header_signature = batch_signature,
            transactions = txns,
            timestamp = int(time.time())
            )
    batch_list_bytes= BatchList(batches=[batch]).SerializeToString()
    #Probably better if send_resquest prints in main.py
    print(send_request(URL,"batches",batch_list_bytes,"application/octet-stream"))

def show(ip : str, wal: str) -> None:
    params = {
        'family' : 'bgt',
        'cmd' : 'show',
        'wallet' : wal
            }
    response = requests.get(f"http://{ip}/run",params=params)
    print(response.text)

def List(ip:str, val_ip : str) -> None:
    params ={
        'family' : 'bgt',
        'url' : val_ip,
        'cmd' : 'list'
            }
    response = requests.get(f"http://{ip}/run",params=params)
    return response.text

